require 'bundler/setup'
require 'rspec/em'

require File.expand_path('../../lib/faye/websocket', __FILE__)
require File.expand_path('../echo_server', __FILE__)
require File.expand_path('../proxy_server', __FILE__)


